/* autodemo.c

Simple demo to demonstrate basic com automation setup under lccwin32. 
Automation Objects can be found using Microsoft's OleView or by searching the Registry.

For example:

HKEY_LOCAL_MACHINE\SOFTWARE\Classes\InternetExplorer.Application

Mike Caetano, Feb 2003
*/
#include <windows.h>
#include <tchar.h>
#include <oleauto.h>
#include <stdio.h>

#ifdef __LCC__
#pragma lib <oleaut32.lib>
#pragma lib <ole32.lib>
#pragma lib <uuid.lib>
#endif


LPTSTR _stdcall ErrorString(DWORD errno)
{

	static TCHAR szMsgBuf[0x1000];

	DWORD msglen = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM
		    , NULL
		    , errno
		    , MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT)
		    , (LPTSTR) &szMsgBuf
		    , 0x1000 * sizeof(TCHAR)
		    , NULL);

	return szMsgBuf;

}

#define THROW(r) \
	do { if ( FAILED((r)) ) \
	   { printf("%s\n", ErrorString((DWORD)(r))); } \
	   } while(0)


int main(void)
{

	HRESULT hr = S_OK;
	IUnknown *pIUnknown = NULL;
	IDispatch *pIDispatch = NULL;

// com
	CoInitialize(NULL);

// get the clsid for the application - the local server
	CLSID clsid;
	WCHAR *lpszProgID  = L"InternetExplorer.Application";
	hr = CLSIDFromProgID(lpszProgID, &clsid);
	THROW(hr);

// dump the clsid for examination
	WCHAR szGUID[64];
	StringFromGUID2(&clsid,szGUID,64);
	printf("%S\n", szGUID);

	BOOL fRunning = TRUE; 	// IE Always runs...

	if ( fRunning  )
	{

		// see Q238610 viz Office interfaces - KB238610

		// A running instance of an Office application
		// must be registered in the Running Object Table (ROT)
		// before it can be attached to using GetActiveObject

		// Else
		// Run-time error '429': ActiveX component can't create object
		// -or-
		// Error: 0x800401e3 "Operation unavailable"

		hr = GetActiveObject(&clsid, NULL, &pIUnknown);
		THROW(hr);

	}

// try again if needed
	if ( NULL == pIUnknown ) {

		hr = CoCreateInstance(
			&clsid,
			NULL,
			CLSCTX_LOCAL_SERVER,
			&IID_IUnknown,
			(void **)&pIUnknown
			    );
		THROW(hr);

	}

// get the dispatch interface
	hr = pIUnknown->lpVtbl->QueryInterface(pIUnknown, &IID_IDispatch, (void **)&pIDispatch);
	THROW(hr);

// list of names for mapping
	WCHAR *rgszNames = L"Name";

	DISPID dispid = 0;

	hr =  pIDispatch->lpVtbl->GetIDsOfNames(
		pIDispatch,
		(REFIID)&IID_NULL,
		&rgszNames,
		1,
		LOCALE_USER_DEFAULT,
		&dispid
		    );
	THROW(hr);

// prep invoke params
	VARIANT varResult;
	VariantInit(&varResult);

	DISPPARAMS args = {
		0, 0, 0, 0 		};
	EXCEPINFO except;
	UINT argErr;

	hr = pIDispatch->lpVtbl->Invoke(
		pIDispatch,
		dispid,
		(REFIID)&IID_NULL,
		LOCALE_USER_DEFAULT,
		DISPATCH_PROPERTYGET,
		&args,
		&varResult,
		&except,
		&argErr
		    );

	THROW(hr);

// dump the results
	printf("%S\n", ( varResult.vt != VT_BSTR ) ? NULL : varResult.bstrVal);

// how to copy a bstr from a variant
	BSTR bstr = ( varResult.vt != VT_BSTR ) ? NULL : varResult.bstrVal;
	DWORD cbDest = SysStringLen(bstr) * sizeof(WCHAR);
	WCHAR *pString = malloc(cbDest);
	memcpy(pString, bstr, cbDest);
	printf("%S\n", pString);
	free(pString);

// clear the variant before it goes out of scope
	hr = VariantClear(&varResult);
	THROW(hr);

// release
	hr = pIDispatch->lpVtbl->Release(pIDispatch);
	THROW(hr);

// release
	hr = pIUnknown->lpVtbl->Release(pIUnknown);
	THROW(hr);

// com
	CoUninitialize();

	return 0;

}
